/**
 *  Client
 *
 *  Entry point into the client portion of sicksync
 */
'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _confConstants = require('../../conf/constants');

var _confConstants2 = _interopRequireDefault(_confConstants);

var _confText = require('../../conf/text');

var _confText2 = _interopRequireDefault(_confText);

var _confEvents = require('../../conf/events');

var _confEvents2 = _interopRequireDefault(_confEvents);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

var _bigSync = require('../big-sync');

var _bigSync2 = _interopRequireDefault(_bigSync);

var FSHelper = _util2['default'].uniqInstance(_confConstants2['default'].FS_TOKEN, require('./fs-helper')),
    WebSocketClient = _util2['default'].uniqInstance(_confConstants2['default'].WS_TOKEN, require('./ws-client')),
    hostname = _os2['default'].hostname(),
    wsEvents = _confEvents2['default'].WS.LOCAL,
    fsEvents = _confEvents2['default'].FS.LOCAL;

function triggerBigSync(project, params, cb) {
    (0, _bigSync2['default'])({
        project: project.project,
        excludes: project.excludes,
        sourceLocation: _util2['default'].ensureTrailingSlash(project.sourceLocation),
        destinationLocation: _util2['default'].ensureTrailingSlash(project.destinationLocation),
        hostname: project.hostname,
        username: project.username
    }, params, cb);
}

function start(config, projects) {
    var foundProjects = _util2['default'].getProjectsFromConfig(config, projects);

    if (_lodash2['default'].isEmpty(foundProjects)) {
        return logProjectsNotFound(foundProjects);
    }

    _lodash2['default'].each(foundProjects, function (project) {
        startProject(config, project);
    });
}

function startProject(config, projectConf) {
    var localLog = _util2['default'].generateLog(projectConf.project, hostname);
    var remoteLog = _util2['default'].generateLog(projectConf.project, projectConf.hostname);
    var sourceLocation = _util2['default'].ensureTrailingSlash(projectConf.sourceLocation);
    var destinationLocation = _util2['default'].ensureTrailingSlash(projectConf.destinationLocation);
    var secret = _util2['default'].getId();

    var fsHelper = new FSHelper({
        sourceLocation: sourceLocation,
        excludes: projectConf.excludes,
        followSymlinks: projectConf.followSymlinks
    });

    var wsClient = new WebSocketClient({
        username: projectConf.username,
        hostname: projectConf.hostname,
        websocketPort: projectConf.websocketPort,
        secret: secret,
        prefersEncrypted: projectConf.prefersEncrypted,
        retryOnDisconnect: config.retryOnDisconnect
    });

    // WS events
    wsClient.on(wsEvents.READY, function () {
        triggerBigSync(projectConf, { debug: config.debug }, function () {
            fsHelper.watch();

            localLog(_confText2['default'].SYNC_ON_CONNECT, projectConf.hostname, projectConf.prefersEncrypted ? 'using' : 'not using', 'encryption');
        });
    });

    wsClient.on(wsEvents.RECONNECTING, _lodash2['default'].partial(_lodash2['default'].ary(localLog, 1), _confText2['default'].SYNC_ON_RECONNECT));

    wsClient.on(wsEvents.DISCONNECTED, function () {
        localLog(_confText2['default'].SYNC_ON_DISCONNECT);
        process.exit();
    });

    wsClient.on(wsEvents.REMOTE_ERROR, function (err) {
        localLog(_confText2['default'].SYNC_ON_REMOTE_LOST, err);
        process.exit();
    });

    wsClient.on(wsEvents.REMOTE_NOT_FOUND, function (err) {
        localLog(_confText2['default'].SYNC_ON_REMOTE_NOT_FOUND, projectConf.hostname, err);
        process.exit();
    });

    wsClient.on(wsEvents.REMOTE_MESSAGE, function (message) {
        // Since WS can be shared amongst projects, filter out
        // any that are not in this project
        if (_lodash2['default'].contains(message, destinationLocation)) {
            remoteLog(message);
        }
    });

    // FS events
    fsHelper.on(fsEvents.CHANGE, function (fileChange) {
        fileChange.destinationpath = destinationLocation + fileChange.relativepath;
        fileChange.subject = 'file';

        localLog('>', fileChange.changeType, fileChange.localpath);

        wsClient.send(fileChange);
    });

    fsHelper.on(fsEvents.LARGE, function () {
        localLog(_confText2['default'].SYNC_ON_LARGE_CHANGE);
        fsHelper.pauseWatch();

        triggerBigSync(projectConf, { debug: config.debug }, function () {
            localLog(_confText2['default'].SYNC_ON_LARGE_CHANGE_DONE);
            fsHelper.watch();
        });
    });
}

function once(config, projects, opts) {
    var foundProjects = _util2['default'].getProjectsFromConfig(config, projects);

    if (_lodash2['default'].isEmpty(foundProjects)) {
        return logProjectsNotFound(foundProjects);
    }

    _lodash2['default'].each(foundProjects, function (project) {
        var localLog = _util2['default'].generateLog(project.project, hostname);

        localLog(_confText2['default'].SYNC_ON_ONCE);

        triggerBigSync(project, {
            dry: opts.dryRun,
            debug: config.debug
        }, _lodash2['default'].partial(localLog, _confText2['default'].SYNC_ON_ONCE_DONE));
    });
}

function logProjectsNotFound(projects) {
    var projectsWanted = projects.length ? projects : process.cwd();

    console.info(_confText2['default'].PROJECT_NOT_FOUND, projectsWanted);
}

exports['default'] = { start: start, once: once };
module.exports = exports['default'];