'use strict';

Object.defineProperty(exports, '__esModule', {
    value: true
});
var _ = require('lodash'),
    fs = require('fs-extra'),
    child = require('child_process'),
    minimatch = require('minimatch'),
    chalk = require('chalk'),
    path = require('path'),
    untildify = require('untildify'),
    constants = require('../conf/constants'),
    text = require('../conf/text'),
    exec = child.exec,
    spawn = child.spawn;

// Returns the path to the sicksync dir
function getSicksyncDir() {
    return untildify(constants.SICKSYNC_DIR);
}

// Return the path to the update json
function getUpdatePath() {
    return getSicksyncDir() + '/' + constants.UPDATE_FILE;
}

// Returns the path the the config file
function getConfigPath() {
    return getSicksyncDir() + '/' + constants.CONFIG_FILE;
}

// Returns the config object if it exists, if not an empty object. Loads/saves from cache where possible
function getConfig() {
    var config = {},
        configPath = getConfigPath();

    if (fs.existsSync(configPath)) {
        config = require(configPath);
    }

    // Return a cloned copy of config to avoid indirect changes
    return _.cloneDeep(config);
}

// Randomly generate a unique ID
function getId() {
    return Math.random().toString(36).substr(2, 9) + Date.now();
}

// Write out the config file with a provided <obj>
function writeConfig(configFile) {
    var configPath = getConfigPath();

    fs.outputFileSync(configPath, JSON.stringify(configFile, null, 4));
    console.info(text.CONFIG_SAVED);
}

// Given a file path, check to see if it's in the excludes array
function isExcluded(filepath, excludes) {
    var result = false;

    excludes.forEach(function (exclude) {
        if (minimatch(filepath, exclude)) result = true;
    });

    return result;
}

// Log messages with Hostname prepended
function generateLog(projectName, hostname) {
    var args = _.slice(arguments);

    // If only one argument it's the hostname
    if (args.length === 1) {
        projectName = null;
        hostname = args[0];
    }

    return function log() {
        var args = [projectName ? chalk.blue('[' + projectName + ']') : '', hostname ? chalk.green('[' + hostname + ']') : ''].concat([].slice.call(arguments));

        console.info.apply(console, args);
    };
}

//
// Rebounce
//
// Takes a desired function, a fallback function, the number of times
// called, and a cool down. Returns a `rebounced` function.
//
// The basic idea is that we want to limit calls to function to a
// certain # of times in a given time-frame. If it breaks over that,
// then we fallback to another function, and halt previous desired calls
function rebounce(primaryFn, secondaryFn, fallOverAmount, coolDown) {
    var timesCalled = 0;
    var timeOutIds = [];
    var fallbackCalled = false;

    return function rebounced() {
        var args = arguments;
        timesCalled++;

        timeOutIds.push(setTimeout(function () {
            timesCalled = 0;
            primaryFn.apply(null, args);
        }, coolDown));

        if (timesCalled >= fallOverAmount) {
            timeOutIds.forEach(clearTimeout);

            /* istanbul ignore else */
            if (!fallbackCalled) {
                timesCalled = 0;
                fallbackCalled = true;
                secondaryFn.apply(null, arguments);
                setTimeout(function () {
                    fallbackCalled = false;
                }, coolDown);
            }
        }
    };
}

function ensureTrailingSlash(path) {
    return path.substring(path.length - 1) === '/' ? path : path + '/';
}

function open(parameter) {
    return exec('open ' + parameter);
}

function toBoolean(param) {
    var lowerParam = param.toLowerCase();
    if (lowerParam.indexOf('y') > -1) {
        return true;
    }

    if (lowerParam.indexOf('n') > -1) {
        return false;
    }

    if (lowerParam === 'true' || lowerParam === 'false') {
        return JSON.parse(lowerParam);
    }

    return false;
}

function setupPrompter(prompt) {
    prompt.message = '';
    prompt.delimiter = '';
    prompt.start();

    return prompt;
}

function shellIntoRemote(remote) {
    return spawn('ssh', ['-tt', remote]);
}

function printLogo() {
    console.info(chalk.blue(fs.readFileSync(path.resolve(__dirname, '../conf/logo.txt')).toString()));
}

function uniqInstance(tokenPath, Constructor) {
    var instances = {};

    return function (args) {
        var token = _.get(args, tokenPath, null);

        if (_.get(instances, token, null)) {
            return instances[token];
        }

        if (!_.isNull(token)) {
            instances[token] = new Constructor(args);
            return instances[token];
        }

        return new Constructor(args);
    };
}

function getProjectFromCwd(config) {
    return _.chain(config.projects).filter(function (project) {
        return _.isEqual(ensureTrailingSlash(untildify(project.sourceLocation)), ensureTrailingSlash(process.cwd()));
    }).value();
}

function getProjectsFromConfig(config, projects) {
    var foundProjects = [];

    if (_.isEmpty(projects)) {
        var cwdProject = getProjectFromCwd(config);

        if (cwdProject) {
            foundProjects = cwdProject;
        }
    }

    _.each(projects, function (project) {
        var projectConf = _.findWhere(config.projects, { project: project });

        if (!_.isEmpty(projectConf)) {
            foundProjects.push(projectConf);
        }
    });

    return foundProjects;
}

exports['default'] = {
    getSicksyncDir: getSicksyncDir,
    getUpdatePath: getUpdatePath,
    getConfigPath: getConfigPath,
    getConfig: getConfig,
    getId: getId,
    writeConfig: writeConfig,
    isExcluded: isExcluded,
    getProjectsFromConfig: getProjectsFromConfig,
    getProjectFromCwd: getProjectFromCwd,
    setupPrompter: setupPrompter,
    shellIntoRemote: shellIntoRemote,
    printLogo: printLogo,
    uniqInstance: uniqInstance,
    open: open,
    toBoolean: toBoolean,
    ensureTrailingSlash: ensureTrailingSlash,
    rebounce: rebounce,
    generateLog: generateLog
};
module.exports = exports['default'];